@@ -7,9 +7,9 @@
 import sys
 from winswitch.util.simple_logger import Logger
 from winswitch.util.simple_logger import set_log_filename, set_log_dir
-from winswitch.util.paths import CLIENT_DIR, CLIENT_LOG
+from winswitch.util.paths import CLIENT_DIR, APPLET_LOG
 set_log_dir(CLIENT_DIR)
-set_log_filename(CLIENT_LOG)
+set_log_filename(APPLET_LOG)
 logger = Logger("applet")
 debug_import = logger.get_debug_import()
 
@@ -186,7 +186,7 @@ class WinSwitchApplet(ClientBase):
 		ClientBase.do_threaded_setup(self)
 		populate_pixmap_lookup()
 		self.startup_progress("prepare UI..", 90)
-	
+
 	def run(self):
 		self.get_root_window()									# early check to ensure that the display is accessible
 		self.wm_util = get_wm_util()
@@ -230,7 +230,7 @@ class WinSwitchApplet(ClientBase):
 			else:
 				if self.tray.getMacDock() is not None:
 					self.create_application_menu()
-					self.preferences_menuitem.set_sensitive(False)					
+					self.preferences_menuitem.set_sensitive(False)
 				else:
 					#give it a simple menu
 					self.menu = gtk.Menu()
@@ -256,8 +256,10 @@ class WinSwitchApplet(ClientBase):
 			self.preferences_menuitem.set_sensitive(True)
 		self.schedule_menu_refresh()
 		def tray_ready():
+			self.sdebug()
 			self.tray.ready()
 			size = self.tray.get_size()
+			self.sdebug("tray size=%s" % str(size))
 			if size:
 				# find the nearest menu size that is lower or equal
 				sizes = MENU_SIZES.keys()[:]
@@ -271,9 +273,9 @@ class WinSwitchApplet(ClientBase):
 							break
 				self.ui_util.default_size = best_size
 				self.slog("set icon default_size=%s from size=%s" % (best_size, size))
-			
+
 		if self.tray:
-			tray_ready()
+			gobject.idle_add(tray_ready)
 		else:
 			self.on_tray_ready.append(tray_ready)
 		ClientBase.settings_ready(self)
@@ -284,6 +286,7 @@ class WinSwitchApplet(ClientBase):
 		if self.local_server:
 			self.local_server.ask = self.ask
 			self.local_server.notify = self.notify
+			self.local_server.selinux_check = self.selinux_enabled_check
 
 
 	#**********************************************************************
@@ -297,6 +300,7 @@ class WinSwitchApplet(ClientBase):
 		return	False
 
 	def selinux_enabled_check(self):
+		self.slog("selinux_enabled_warning=%s" % self.settings.selinux_enabled_warning)
 		if not LINUX or self.settings.selinux_enabled_warning<=0 or is_selinux_checked():
 			return
 		def selinux_callback(selinux_enabled, selinux_enforcing, warning_bool, info):
@@ -526,62 +530,104 @@ class WinSwitchApplet(ClientBase):
 		"""
 		Overrides start_mdns_listener on ClientBase to add an alert box telling the user about the missing feature and how to enable it.
 		"""
-		while self.settings.mdns_listener:
-			ClientBase.start_mdns_listener(self)
-			if self.mdns_listener:
-				return
+		ClientBase.start_mdns_listener(self)
+		if self.mdns_listener:
+			return
 
-			msg = "mDNS is disabled! Automatic discovery is OFF!"
-			if self.settings.ignore_missing_mdns:
-				self.slog(msg)
-				return
+		msg = "mDNS is disabled! Automatic discovery is OFF!"
+		if self.settings.ignore_missing_mdns:
+			self.slog(msg)
+			return
 
-			self.serr(msg)
-			text = "In order to automatically detect local servers on your local network,\n"
-			text += "%s uses Apple's 'Bonjour' protocol.\n" % APPLICATION_NAME
-			text += "This component seems to be missing or it is not running."
-			text += "\nInstall it first then click retry."
-			mdns_url = None
-			mdns_url_label = None
-			if WIN32:
-				text += "\n\n(you only need 'bonjour' and not all the other optional\ncomponents Apple will try to make you install)"
-				mdns_url = "http://apple.com/support/downloads/bonjourforwindows.html"
-				mdns_url_label = "Download it from apple.com"
+		self.serr(msg)
+		text = "In order to automatically detect local servers on your local network,\n"
+		text += "%s uses Apple's 'Bonjour' protocol.\n" % APPLICATION_NAME
+		text += "This component seems to be missing or it is not running."
+		text += "\nInstall it first then click retry."
+		mdns_url = None
+		mdns_url_label = None
+		install_action = None
+		install_tooltip = None
+		if WIN32:
+			text += "\n\n(you only need 'bonjour' and not all the other optional\ncomponents Apple will try to make you install)"
+			mdns_url = "http://apple.com/support/downloads/bonjourforwindows.html"
+			mdns_url_label = "Download it from apple.com"
+		else:
+			if OSX:
+				mdns_url = "http://developer.apple.com/networking/bonjour/index.html"
+				mdns_url_label = "See apple.com for more information"
 			else:
-				if OSX:
-					mdns_url = "http://developer.apple.com/networking/bonjour/index.html"
-					mdns_url_label = "See apple.com for more information"
-				else:
+				from winswitch.util.distro_packaging_util import get_distro_helper
+				install_action = get_distro_helper().get_install_action("avahi")
+				install_tooltip = "Install now via %s" % get_distro_helper().manager
+				if not install_action:
 					text += "\nOn most *nix systems, this means installing and starting avahi"
 					text += "\nand also installing its python bindings."
 
-			dialog = gtk.Dialog('Automatic Discovery Feature Missing',
-                 None,  # the window that spawned this dialog
-                 gtk.DIALOG_MODAL,
-				 ("Retry", 10077, gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
-			dialog.vbox.set_spacing(10)
-			dialog.vbox.pack_start(self.ui_util.make_label('Automatic Discovery Feature Missing', "sans 14"))
-			dialog.vbox.pack_start(gtk.Label(text))
-			if mdns_url:
-				mdns_url_button = gtk.LinkButton(mdns_url, mdns_url_label)
-				mdns_url_button.set_relief(gtk.RELIEF_NONE)
-				def download_mdns(*args):
-					self.sdebug(None, *args)
-					self.open_url(mdns_url)
-				mdns_url_button.connect("clicked", download_mdns)
-				dialog.vbox.pack_start(mdns_url_button)
-			dismiss = gtk.CheckButton("Do not show this message again")
-			dialog.vbox.pack_start(dismiss)
-			dialog.show_all()
-			result = dialog.run()
-			dialog.destroy()
-			if result == gtk.RESPONSE_CLOSE:
-				ignore = dismiss.get_active()
-				if ignore:
-					self.settings.ignore_missing_mdns = ignore
-					modify_settings(self.settings, ["ignore_missing_mdns"])
-				return
+		dialog = gtk.Window()
+		dialog.set_title('Automatic Discovery Feature Missing')
+		dialog.set_destroy_with_parent(True)
+		dialog.set_resizable(False)
+		dialog.set_decorated(True)
+		dialog.set_icon(icons.get(get_ui_util().get_best_colour_scheme_icon_name()))
+		dialog.set_position(gtk.WIN_POS_CENTER)
+		dialog.set_border_width(10)
 
+		def cancel_dialog(*args):
+			ignore = dismiss.get_active()
+			self.slog("closed, ignore=%s" % ignore)
+			if ignore:
+				self.settings.ignore_missing_mdns = ignore
+				self.slog("updating settings")
+				modify_settings(self.settings, ["ignore_missing_mdns"])
+			dialog.hide_all()
+			dialog.destroy()
+		def retry(*args):
+			self.slog(None, *args)
+			dialog.hide_all()
+			ClientBase.start_mdns_listener(self)
+			if self.mdns_listener:
+				dialog.destroy()
+			else:
+				dialog.show_all()
+
+		vbox = gtk.VBox(False, 0)
+		vbox.set_spacing(10)
+		dialog.add(vbox)
+		dialog.set_geometry_hints(vbox)
+
+		vbox.pack_start(self.ui_util.make_label('Automatic Discovery Feature Missing', "sans 14"))
+		vbox.pack_start(gtk.Label(text))
+		if install_action:
+			def install_mdns(*args):
+				self.slog(None, *args)
+				install_action(retry)
+			mdns_install_button = self.ui_util.make_imagebutton("install", "download", install_tooltip, install_mdns)
+			vbox.pack_start(mdns_install_button)
+		elif mdns_url:
+			mdns_url_button = gtk.LinkButton(mdns_url, mdns_url_label)
+			mdns_url_button.set_relief(gtk.RELIEF_NONE)
+			def download_mdns(*args):
+				self.sdebug(None, *args)
+				self.open_url(mdns_url)
+			mdns_url_button.connect("clicked", download_mdns)
+			vbox.pack_start(mdns_url_button)
+		dismiss = gtk.CheckButton("Do not show this message again")
+		vbox.pack_start(dismiss)
+
+		#buttons:
+		hbox = gtk.HBox(True, 10)
+		cancel_button = self.ui_util.make_imagebutton("Ignore", "remove", "Continue without mDNS", cancel_dialog, 24)
+		hbox.pack_start(cancel_button)
+		retry_button = self.ui_util.make_imagebutton("Retry", "retry", "Retry to start the mDNS feature", retry, 24)
+		hbox.pack_start(retry_button)
+		vbox.pack_start(hbox)
+
+		#accel / close:
+		dialog.connect('delete_event', cancel_dialog)
+		self.ui_util.add_close_accel(dialog, cancel_dialog)
+
+		dialog.show_all()
 
 
 
@@ -954,14 +1000,14 @@ class WinSwitchApplet(ClientBase):
 				""" modifier action is to ask the user: show a dialog """
 				def session_start_options_start(session_type, screen_size, opts, bookmark):
 					self.do_start_session(server, command, session_type, screen_size, opts, bookmark)
-				ask_options = SessionStartOptions(server, self.client_utils, command, command.type, session_start_options_start, False)
+				ask_options = SessionStartOptions(server, self.client_utils, command, command.type, session_start_options_start, None, False)
 				ask_options.show()
 				return	True
 		return ClientBase.default_start_session(self, server, command)
 
 
 
-	def start_menu_callback(self, widget, server, server_command, session_type=None, screen_size=None, opts={}):
+	def start_menu_callback(self, widget, server, server_command, session_type=None, screen_size=None, opts=None):
 		"""
 		Calls either SessionStartOptions() if session_type and preferred_session_type are missing or if "modifier key" was held,
 		or do_start_session() otherwise.
@@ -973,14 +1019,11 @@ class WinSwitchApplet(ClientBase):
 			else:
 				session_type = server.preferred_session_type
 		if self.is_modifier_set() or not session_type:
-			def session_start_options_start(session_type, screen_size, opts, bookmark):
-				self.do_start_session(server, server_command, session_type, screen_size, opts, bookmark)
-			ask_options = SessionStartOptions(server, self.client_utils, server_command, server_command.type, session_type, session_start_options_start, True)
+			def session_start_options_start(session_type, screen_size, actual_opts, bookmark):
+				self.do_start_session(server, server_command, session_type, screen_size, actual_opts, bookmark)
+			ask_options = SessionStartOptions(server, self.client_utils, server_command, server_command.type, session_type, session_start_options_start, opts, True)
 			ask_options.show()
 		else:
-			display_util = self.get_remote_util(session_type)
-			if display_util:
-				display_util.add_default_options_values(opts, server, server_command)
 			self.do_start_session(server, server_command, session_type, screen_size, opts, False)
 
 	def do_start_menu_dialog(self, server, command_type):
@@ -988,8 +1031,9 @@ class WinSwitchApplet(ClientBase):
 		Show a dialog where the user can select the command to start.
 		(as well as the protocol to use, as per start_menu_callback)
 		"""
-		dialog = SessionStartDialog(self.servers, server, self.client_utils, command_type,
-							lambda _server, command, session_type, screen_size, opts, bookmark : self.do_start_session(_server, command, session_type, screen_size, opts, bookmark))
+		def dialog_do_start(_server, command, session_type, screen_size, opts, bookmark):
+			self.do_start_session(_server, command, session_type, screen_size, opts, bookmark)
+		dialog = SessionStartDialog(self.servers, server, self.client_utils, command_type, dialog_do_start)
 		self.sdebug("dialog=%s" % dialog, server, command_type)
 		dialog.show()
 
@@ -1141,10 +1185,10 @@ class WinSwitchApplet(ClientBase):
 			self.sdebug("last_menu_update=%s, now=%s" % (self.last_menu_update, now))
 		self.last_menu_update = now
 		if self.tray:
-			if self.tray.getMacDock() is not None:
-				return self.create_mac_menu(self.tray.getMacDock())
-			else:
-				return self.create_tray_menu()
+			macdock = self.tray.getMacDock()
+			if macdock:
+				return self.create_mac_menu(macdock)
+			return self.create_tray_menu()
 
 	def create_mac_menu(self, dock):
 		self.sdebug(None, dock)
@@ -1732,8 +1776,9 @@ class WinSwitchApplet(ClientBase):
 		if server.allow_custom_commands:
 			#custom command dialog
 			def custom_command_dialog(*args):
-				custom_dialog = SessionCustomCommandDialog(server, self.client_utils, server.preferred_session_type,
-						lambda server,server_command,session_type,screen_size,opts,bookmark : self.do_start_session(server, server_command, session_type, screen_size, opts, bookmark))
+				def custom_dialog_do_start(server,server_command,session_type,screen_size,opts,bookmark):
+					self.do_start_session(server, server_command, session_type, screen_size, opts, bookmark)
+				custom_dialog = SessionCustomCommandDialog(server, self.client_utils, server.preferred_session_type, custom_dialog_do_start)
 				custom_dialog.show()
 			custom_launch = self.ui_util.menuitem("Custom Command", icons.get("command_prompt"), "Enter a specific command to run", custom_command_dialog)
 			sm.append(custom_launch)
@@ -1839,7 +1884,7 @@ class WinSwitchApplet(ClientBase):
 				actions.append(("Detach", "Detach this session", icons.get("pause"), lambda x : self.detach_session(x, server, session),
 						connected))
 				actions.append(("Re-connect", "Re-connect to this session (will refresh the display)", icons.get("retry"), lambda x : self.reconnect_session(x, server, session),
-						compatible and connected))
+						compatible and connected and server.is_connected()))
 			#actions.append(("Terminate", "Terminate this session", icons.get("close"), lambda x : self.close_session(x, server, session),
 			#			connected))
 
@@ -1996,18 +2041,21 @@ def main():
 		sys.exit(1)
 
 	exit_code = 0
-	try:
-		winswitch.run()
-		logger.slog("run() ended")
-	except Exception, e:
-		logger.serr("caught exception, closing applet", e)
-		exit_code = 1
+	if not winswitch.contact_existing_client():
 		try:
-			winswitch.cleanup()
+			winswitch.run()
+			logger.slog("run() ended")
 		except Exception, e:
-			logger.serror("exception during cleanup()", e)
-	logger.slog("exit code=%d, waiting for all other threads to exit - this may take a few seconds" % exit_code)
-	dump_threads()
+			logger.serr("caught exception, closing applet", e)
+			exit_code = 1
+			try:
+				winswitch.cleanup()
+			except Exception, e:
+				logger.serror("exception during cleanup()", e)
+		logger.slog("exit code=%d, waiting for all other threads to exit - this may take a few seconds" % exit_code)
+		dump_threads()
+	from winswitch.util.simple_logger import rotate_log_file
+	rotate_log_file()
 	return	exit_code
 
 if __name__ == "__main__":
