@@ -183,7 +183,9 @@ class ServerLineConnection:
 
 	def do_send_xmodmap(self):
 		self.slog("sending %s bytes of xmodmap_keys and %s bytes of xmodmap_modifiers to %s" % (len(self.settings.xmodmap_keys or ""), len(self.settings.xmodmap_modifiers or ""), self.server))
-		self.handler.send_xmodmap(self.settings.xmodmap_keys, self.settings.xmodmap_modifiers)
+		self.handler.send_xmodmap(self.settings.xmodmap_keys, self.settings.xmodmap_modifiers,
+								self.settings.xkbmap_print, self.settings.xkbmap_query,
+								self.settings.xkbmap_layout, self.settings.xkbmap_variant)
 		self.server.xmodmap_sent = True
 
 
@@ -329,14 +331,24 @@ class ServerLineConnection:
 				self.request_session_icon(session, False)
 				self.request_session_icon(session, True)
 			""" pre-connect to local server sessions: """
-			if self.server.local and session.preload:
-				self.slog("local server and preload session, will pre-connect", *args)
-				callLater(0.5, self.pre_connect, session)
+			#only do this on posix
+			if self.schedule_preconnect(session):
+				pass
 			elif was_pre_connected_here and session.preload is False:
 				session.pre_connected = False
 				self.slog("pre-connected session has now started - sound may need connecting", *args)
 				self.server_link.may_connect_sound(session)
 
+	def schedule_preconnect(self, session):
+		if not self.server.local:
+			return	False			#only pre-connect to local servers for now..
+		if not session.preload:
+			return	False			#pre-connect only makes sense for "preload" sessions
+		if session.session_type=="xpra" and os.name!="posix":
+			return	False			#FIXME: xpra preloading not available on non-posix systems (shows tray)
+		callLater(1.0, self.pre_connect, session)
+		return True
+
 	def pre_connect(self, session):
 		self.slog(None, session)
 		def session_pre_connected(*args):
@@ -382,6 +394,8 @@ class ServerLineConnection:
 						settings.open_urls, settings.open_files,
 						"pygtk",
 						settings.supports_virtualbox,
+						self.handler.binencode(settings.xkbmap_query),
+                        settings.xkbmap_layout, settings.xkbmap_variant
 						])
 
 	def do_version(self, proto_ver, app_ver, info):
