@@ -114,7 +114,7 @@ else:
         except Exception, e:
             print("error loading %s: %s" % (lib, e))
     if not _libdnssd:
-        raise Exception("Failed to load dns library: %s" % str(libs))
+        raise ImportError("Failed to load dns library: %s" % str(libs))
     _CFunc = ctypes.CFUNCTYPE
 
 
