@@ -605,7 +605,7 @@ class WinSwitchClientChannel(twisted_protocols_basic.LineReceiver):
 	def logged_in_ok(self, user):
 		config = self.controller.config
 		self.add_authenticated_handlers()
-		self.controller.add_local_user(user.username, user.preferred_session_type)
+		self.controller.add_local_user(user.username, user.uuid, user.preferred_session_type)
 		self.sdebug("username=%s, uuid=%s"  % (user.username, user.uuid), user)
 		mod = self.controller.config.add_user(user)
 		self.user = user
@@ -928,8 +928,14 @@ class WinSwitchClientChannel(twisted_protocols_basic.LineReceiver):
 				delete_if_exists(filename)
 				return
 			save_binary_file(filename, xmodmap_data)
-		#tell the server to apply the new xmodmap to all relevant sessions
-		self.controller.set_xmodmap(self.user)
+		#optional fields introduced in 0.12.16:
+		if len(args)>=4:
+			e_xkbmap_print, e_xkbmap_query, xkbmap_layout, xkbmap_variant = args[:4]
+			self.user.xkbmap_print = bindecode(e_xkbmap_print)
+			self.user.xkbmap_query = bindecode(e_xkbmap_query)
+			self.user.xkbmap_layout = xkbmap_layout
+			self.user.xkbmap_variant = xkbmap_variant
+		self.controller.set_new_user_keymap(self.user)
 
 
 	def add_mount_point(self, protocol, namespace, host, port, path, fs_type, auth_mode, username, password, comment, options_str):
