@@ -34,7 +34,7 @@ from winswitch.util.common import visible_command, get_machine_id, is_laptop, cs
 debug_import("selinux_util")
 from winswitch.util.selinux_util import check_selinux, is_selinux_checked
 debug_import("process_util")
-from winswitch.util.process_util import exec_nopipe, register_sigusr_debugging
+from winswitch.util.process_util import twisted_exec, register_sigusr_debugging
 debug_import("config")
 from winswitch.util.config import get_local_server_config, save_local_server_config, load_session_from_file, load_object_from_properties
 debug_import("file_io")
@@ -364,7 +364,7 @@ class WinSwitchServer:
 		if self.config.mDNS_publish:
 			self.start_mdns_publisher()
 
-	def load_existing_session_files(self):
+	def load_existing_session_files(self, enabled_session_types):
 		#only works for non-root...
 		sessions_dir = get_sessions_dir(USERNAME, USER_ID==0)			#directory containing all the session directories
 		all_files = os.listdir(sessions_dir)
@@ -377,6 +377,9 @@ class WinSwitchServer:
 			if not is_valid_file(session_filename):
 				continue
 			session = load_session_from_file(session_filename, True, ServerSession)
+			if session.session_type not in enabled_session_types:
+				self.sdebug("session type %s is not enabled, discarded session file '%s'" % (session.session_type, f))
+				continue
 			if session is None or not session.display:
 				self.sdebug("cowardly ignoring session file %s which does not have a display - please remove manually" % session_filename)
 				continue
@@ -400,7 +403,6 @@ class WinSwitchServer:
 
 	def assign_virt_utils(self):
 		from winswitch.virt.vnc_server_util import VNCServerUtil
-		reloaded_sessions = self.load_existing_session_files()
 
 		virt_args = (self.config, self.add_new_session, self.send_remove_session, self.update_session_status, self.session_failed)
 		if self.config.supports_xpra:
@@ -446,7 +448,8 @@ class WinSwitchServer:
 				needs_VNC()
 		#initialize them
 		for util in set(self.virt_utils.values()):
-			util.load_config()
+			if util.enabled:
+				util.load_config()
 			if util.enabled:
 				util.init()
 			else:
@@ -459,20 +462,24 @@ class WinSwitchServer:
 		self.virt_utils = new_utils
 
 		#now we can delete the dead sessions:
+		reloaded_sessions = self.load_existing_session_files(self.virt_utils.keys())
 		for session in reloaded_sessions:
 			if session.status==Session.STATUS_CLOSED:
 				self.session_cleanup(session)
 		#detect existing sessions:
 		for util in set(self.virt_utils.values()):
+			if not util.enabled:
+				continue
 			disk_sessions = self.live_filtered_sessions(reloaded_sessions, util.session_type)
 			try:
 				util.detect_existing_sessions(disk_sessions)
 			except Exception, e:
 				self.serr("error detecting sessions with %s, disk_sessions=%s" % (util, disk_sessions), e)
 		for util in set(self.virt_utils.values()):
-			util.setup_directory_watcher()
+			if util.enabled:
+				util.setup_directory_watcher()
 		self.slog("=%s" % self.virt_utils)
-		#find the sessions that have not been claimed:
+
 
 	def session_cleanup(self, session):
 		virt = self.get_virt(session)
@@ -524,16 +531,17 @@ class WinSwitchServer:
 			self.config.touch()
 		def stop_vnc_server(*args):
 			self.slog("pid=%s" % pid, *args)
-			from winswitch.util.process_util import kill_pid
-			try :
-				killed = kill_pid(pid)
-			except Exception, e:
-				self.serr("failed to kill pid=%s" % pid, e, *args)
-			self.slog("killed(%s)=%s" % (pid,killed), *args)
+			from winswitch.util.process_util import send_kill_signal
+			killed = send_kill_signal(pid)
+			self.debug("killed(%s)=%s" % (pid,killed), *args)
 			if not killed:
 				args_list = ['%s' % self.config.winvnc_command, '-stop']
 				self.slog("failed to stop vnc server with kill, trying with: %s" % str(args_list))
-				exec_nopipe(args_list)
+				def winvnc_stopped(*args):
+					self.slog(None, *args)
+				def winvnc_not_stopped(*args):
+					self.serror(None, *args)
+				twisted_exec(args_list, winvnc_stopped, winvnc_not_stopped)
 
 		self.ask('VNC port conflict',
 						"The application '%s' is already using the VNC port\n"
@@ -620,12 +628,12 @@ class WinSwitchServer:
 		self.slog(None, args)
 		self.schedule_reload_server_start_menu()
 
-	def add_local_user(self, username, pref_session=None):
-		self.sdebug(None, username, pref_session)
+	def add_local_user(self, username, uuid=None, pref_session=None):
+		self.sdebug(None, username, uuid, pref_session)
 		if not username:
 			return
 		for util in set(self.virt_utils.values()):
-			util.add_local_user(username)
+			util.add_local_user(username, uuid)
 
 	def start_mdns_publisher(self):
 		"""
@@ -650,15 +658,20 @@ class WinSwitchServer:
 				try:
 					from winswitch.net.avahi_publisher import AvahiPublishers
 					self.mdns_publisher = AvahiPublishers(self.listen_on, service_name, MDNS_TYPE, txt)
-				except Exception, e:
-					self.serr("failed to load avahi listener", e)
+				except ImportError, e:
+					self.serror("failed to load avahi publisher: %s" % e)
 			if self.mdns_publisher is None:
-				from winswitch.net.twisted_bonjour_publisher import TwistedBonjourPublishers
-				self.mdns_publisher = TwistedBonjourPublishers(reactor, self.listen_on, service_name, MDNS_TYPE, txt)
-			callFromThread(self.mdns_publisher.start)
+				try:
+					from winswitch.net.twisted_bonjour_publisher import TwistedBonjourPublishers
+					self.mdns_publisher = TwistedBonjourPublishers(reactor, self.listen_on, service_name, MDNS_TYPE, txt)
+				except ImportError, e:
+					self.serror("failed to load pybonjour publisher: %s" % e)
+			if self.mdns_publisher:
+				callFromThread(self.mdns_publisher.start)
+			else:
+				self.serror("if you do not wish to use mDNS, you may want to disable it in the server's config file (set mDNS_publish to False)")
 		except Exception, e:
 			self.serr("mDNS error: %s, mDNS is now disabled! Automatic discovery is OFF!" % e, e)
-			self.serror("if you do not wish to use it, you may want to disable it in the server's config file (set mDNS_publish to False)")
 
 	def get_mdns_text_record(self):
 		text_record = {"ID":self.config.ID}
@@ -941,13 +954,13 @@ class WinSwitchServer:
 			session.capture_pending = False
 		util.capture_display(session, send_new_capture_data, capture_err)
 
-	def set_xmodmap(self, user):
+	def set_new_user_keymap(self, user):
 		""" find all the sessions in use by this user and apply the xmodmap """
 		sessions = self.config.get_connected_sessions(user.uuid, or_connecting=True)
 		self.log("(...) will now update these connected/connecting sessions with the new xmodmap: %s" % csv_list(sessions))
 		for session in sessions:
 			virt = self.get_virt(session)
-			virt.set_xmodmap(session, user)
+			virt.set_keyboard_mappings(session, user)
 
 	def add_new_session(self, session):
 		"""
