@@ -95,7 +95,7 @@ class ConfigureAppletWindow(TabbedConfigureWindow):
 
 	def add_protocol_file_select(self, table, session_type, name, patterns, any_file=True, tool_name=None, x_offset=0, x_factor=1, row_inc=True):
 		dh = get_distro_helper()
-		install_command=dh.get_install_action(session_type)
+		install_command = dh.get_install_action(session_type)
 		pagename = session_type
 		if (session_type==WINDOWS_TYPE):
 			pagename = "rdp"
@@ -129,10 +129,8 @@ class ConfigureAppletWindow(TabbedConfigureWindow):
 					file_input_changed()
 			def dh_install(*args):
 				""" call the install command, then try dh_locate again """
-				from winswitch.util.process_util import twisted_exec
-				def dh_install_err(err):
-					self.slog(err)
-				twisted_exec(install_command, dh_locate, dh_install_err)
+				self.sdebug("install_command=%s" % install_command)
+				install_command(dh_locate)
 			def dh_setup(*args):
 				located = dh.find_command(session_type)
 				if located:
