@@ -289,8 +289,6 @@ def get_session_password_filename(display, username=USERNAME, as_root=False):
 	return do_get_session_filename(display, username, as_root, ext=".pass")
 def get_session_wrapper_log_filename(display, username=USERNAME, as_root=False):
 	return do_get_session_filename(display, username, as_root, "command", ".log")
-def get_session_Xdummy_log_filename(display, username=USERNAME, as_root=False):
-	return do_get_session_filename(display, username, as_root, "Xdummy", ".log")
 def get_session_pulseaudio_log_filename(display, username=USERNAME, as_root=False):
 	return do_get_session_filename(display, username, as_root, "pulse", ".log")
 def get_session_xmodmap_filename(display, username=USERNAME, as_root=False, is_modifiers=False):
@@ -367,7 +365,7 @@ def load_object_from_properties(filename, instance_type, keys_in=None, extra_key
 	is_new_version = False 
 	if not saved_version or saved_version!=WINSWITCH_VERSION:
 		if warn_on_missing_keys:
-			logger.slog("different version: file is from %s, current version is %s" % (saved_version, WINSWITCH_VERSION), filename, instance_type, keys_in, extra_keys, constructor, newerversion_constructor, ignored_keys, warn_on_missing_keys, warn_on_extra_keys)
+			logger.sdebug("different version: file is from %s, current version is %s" % (saved_version, WINSWITCH_VERSION), filename, instance_type, keys_in, extra_keys, constructor, newerversion_constructor, ignored_keys, warn_on_missing_keys, warn_on_extra_keys)
 		is_new_version = True
 	try:
 		if is_new_version and newerversion_constructor:
