@@ -188,7 +188,7 @@ def do_load_start_menu(ignored_commands, whitelist_commands, ignored_categories=
 		data_dirs = os.environ.get("XDG_DATA_DIRS", "/usr/local/share/:/usr/share/").split(":")
 		menu_home_dirs = data_dirs + [xdg_data_home]
 		menu_dirs = [os.path.join(x, "applications") for x in menu_home_dirs]
-		logger.error(sig+" no files found in %s, using system locations: %s" % (menu_dir, menu_dirs))
+		logger.log(sig+" no files found in %s, using system locations: %s" % (menu_dir, menu_dirs))
 		server_commands = load_menu_files_from_dirs(menu_dirs, ".desktop", True, ignored_commands, whitelist_commands,
 												ignored_directories=ignored_directories,
 												file_type=ServerCommand.COMMAND)
@@ -249,7 +249,7 @@ def load_menu_files(menu_dir, extension, req_cmd, ignored_commands=None, whiteli
 					ignored_directories=[],
 					file_type=ServerCommand.COMMAND):
 	sig = msig(menu_dir, extension, req_cmd, ignored_commands, whitelist_commands, ignored_in, recurse, ignored_directories, file_type)
-	logger.log(sig)
+	logger.debug(sig)
 	menu_files = []
 	if not os.path.exists(menu_dir) or not os.path.isdir(menu_dir):
 		return	menu_files
@@ -490,7 +490,6 @@ def load_menu_file(menu_dir, filename, req_cmd, ignored_commands=None, whitelist
 	sc.uses_sound_out = uses_sound_out
 	sc.uses_sound_in = uses_sound_in
 	sc.uses_video = uses_video
-	logger.log("load_menu_file(..) %s.uses_video=%s" % (full_path, uses_video))
 	sc.run_once_only = run_once_only
 	pos = filename.rfind(".")
 	if pos>0:
