@@ -4,6 +4,11 @@
 # Copyright (c) 2009-2012 Antoine Martin <antoine@nagafix.co.uk>
 # Window-Switch is released under the terms of the GNU GPL v3
 
+from winswitch.util.simple_logger import Logger, msig, set_loggers_debug
+logger=Logger("process_util", log_colour=Logger.CYAN)
+debug_import = logger.get_debug_import()
+
+debug_import("os/sys/time/signal/subprocess/threading/collections")
 import os
 import sys
 import time
@@ -12,17 +17,22 @@ import subprocess
 import threading
 from collections import deque
 
+debug_import("twisted.internet.error")
 from twisted.internet.error import ProcessDone, ProcessTerminated
+debug_import("twisted.python.failure")
 from twisted.python.failure import Failure
+debug_import("winswitch.twisted.HiddenSpawnedProcess")
 from winswitch.twisted.HiddenSpawnedProcess import HiddenSpawnedProcess
 
+debug_import("globals")
 from winswitch.globals import SUBPROCESS_CREATION_FLAGS, WIN32
-from winswitch.util.simple_logger import Logger, msig, set_loggers_debug
+debug_import("util.common")
 from winswitch.util.common import visible_command, escape_newlines, csv_list, is_valid_file, is_valid_dir
+debug_import("util.main_loop")
 from winswitch.util.main_loop import callLater
+debug_import("util.consts")
 from winswitch.consts import NOTIFY_ERROR
 
-logger=Logger("process_util", log_colour=Logger.CYAN)
 logger.slog("using twisted ProcessProtocol=%s" % HiddenSpawnedProcess)
 
 BUFFER_PREVIOUS_LINES = 64
@@ -47,7 +57,7 @@ class DaemonPidWrapper:
 	def stop(self):
 		self.slog("wrapper=%s" % self)
 		if (self.pid>1):
-			kill_pid(self.pid, sigint=True)
+			send_kill_signal(self.pid)
 
 
 def exec_daemonpid_wrapper(cmd, env=None, shell=False, cwd=None):
@@ -257,7 +267,10 @@ class LineProcessProtocolWrapper(HiddenSpawnedProcess):
 				self.returncode = 0
 			elif isinstance(process_info, ProcessTerminated):
 				self.returncode = process_info.exitCode
-				self.serror("Failure (command=%s), exitCode=%s" % (str(self.command), self.returncode), reason)
+				if self.log_lifecycle:
+					self.serror("Failure (command=%s), exitCode=%s" % (str(self.command), self.returncode), reason)
+				else:
+					self.sdebug("Failure (command=%s), exitCode=%s" % (str(self.command), self.returncode), reason)
 		self.exit()
 
 	def processEnded(self, reason):
@@ -426,7 +439,7 @@ def twisted_exec(cmd, ok_callback, err_callback=None, path=None, env=None, log_l
 
 def kill_daemon(pid, fd_file):
 	if is_daemon_alive(pid, fd_file):
-		kill_pid(pid)
+		send_kill_signal(pid)
 
 def is_daemon_alive(pid, fd_file, fds=None):
 	"""
@@ -509,38 +522,42 @@ def get_command_for_pid(pid):
 	return	("%s" % data).replace("\x00", "")
 
 
-no_oskill_warned = False
-def kill_pid(pid, sigint=True):
-	if hasattr(os, "kill"):
-		logger.slog("using os.kill", pid)
-		try:
-			if sigint:
-				os.kill(pid , signal.SIGINT)
-			else:
-				os.kill(pid , signal.SIGKILL)
-			return	True
-		except Exception, e:
-			logger.serr(None, e, pid)
-	else:
-		global no_oskill_warned
-		if not no_oskill_warned:
-			logger.slog("os.kill not available!", pid)
-			no_oskill_warned = True
 
-	if WIN32:
+def win32_TerminateProcess(pid):
+	handle = None
+	try:
+		import win32api		#@UnresolvedImport
 		PROCESS_TERMINATE = 1
-		handle = None
+		handle = win32api.OpenProcess(PROCESS_TERMINATE, False, pid)
+		win32api.TerminateProcess(handle, -1)
+		win32api.CloseHandle(handle)
+		logger.slog("used win32api.CloseHandle(%s)" % handle, pid)
+		return	True
+	except Exception, e:
+		logger.serr("win32api.CloseHandle(%s) failed" % handle, e, pid)
+		return	False
+
+
+def send_kill_signal(pid, kill_signal=signal.SIGTERM):
+	global no_oskill_warned
+	logger.slog(None, pid, kill_signal)
+	if pid is None or pid<1:
+		return False
+
+	#only supported on win32 since Python 2.7
+	if hasattr(os, "kill"):
+		logger.sdebug("using os.kill", pid, kill_signal)
 		try:
-			import win32api		#@UnresolvedImport
-			handle = win32api.OpenProcess(PROCESS_TERMINATE, False, pid)
-			win32api.TerminateProcess(handle, -1)
-			win32api.CloseHandle(handle)
-			logger.slog("used win32api.CloseHandle(%s)" % handle, pid)
+			os.kill(pid , kill_signal)
 			return	True
 		except Exception, e:
-			logger.serr("win32api.CloseHandle(%s) failed" % handle, e, pid)
-			return	False
-	return	False
+			logger.serror("%s" % e, pid, kill_signal)
+			return False
+	elif WIN32:
+		return win32_TerminateProcess(pid)
+	else:
+		os.system("kill -%s %d" % (kill_signal, pid))
+		return True
 
 
 def subprocess_terminate(proc, check_returncode=True):
@@ -559,7 +576,7 @@ def subprocess_terminate(proc, check_returncode=True):
 			logger.serror("illegal pid: %s" % proc.pid, proc, check_returncode)
 			return
 
-		kill_pid(proc.pid)
+		send_kill_signal(proc.pid)
 	except Exception, e:
 		logger.serr("pid=%s" % proc.pid, e, proc, check_returncode)
 
