@@ -4,9 +4,7 @@
 # Copyright (c) 2009-2012 Antoine Martin <antoine@nagafix.co.uk>
 # Window-Switch is released under the terms of the GNU GPL v3
 
-import time
-import re
-import sys
+import os, time, re, sys
 
 from winswitch.consts import XPRA_TYPE, DEFAULT_LAN_SPEED, MODEM_56K_SPEED, DSL_256K_SPEED
 from winswitch.globals import USERNAME, OSX, WIN32
@@ -15,6 +13,7 @@ from winswitch.util.file_io import get_client_session_password_file
 from winswitch.virt.client_util_base import ClientUtilBase
 from winswitch.virt.xpra_common import xpra_cmd
 from winswitch.virt.options_common import CLIPBOARD, ENCODING, KEYBOARD_SYNC, READ_ONLY, MMAP
+from winswitch.objects.server_command import ServerCommand
 from winswitch.util.main_loop import callLater
 
 XPRA_DEBUG_OPTION="--debug-xpra" in sys.argv
@@ -46,6 +45,7 @@ class	XpraClientBase(ClientUtilBase):
 		self.slog(None, server, client, session)
 		try:
 			if client.handler.remote_app_version>=[0,12,10]:
+				session.kill_client = True		#prevent error in client process exit path
 				processes = session.processes
 				client_pid = session.client_pid
 				client.handler.send_disconnect_session(session.ID)
@@ -61,7 +61,7 @@ class	XpraClientBase(ClientUtilBase):
 	def get_default_compression(self, server, server_command):
 		""" guess what the default compression should be: use line speed for remote video commands, otherwise lossless """
 		c = 0
-		if server_command and server_command.uses_video and server.supports_xpra_encodings:
+		if server_command and server.supports_xpra_encodings and (server_command.uses_video or server_command.type==ServerCommand.DESKTOP):
 			self.sdebug("non local server and command uses_video", server, server_command)
 			for x in ["x264", "vpx", "jpeg"]:
 				if x in server.supports_xpra_encodings:
@@ -111,15 +111,20 @@ class	XpraClientBase(ClientUtilBase):
 							xpra_args_list += ["--auto-refresh-delay=%s" % self.get_auto_refresh_delay(q, server.line_speed)]
 				except:
 					pass
+		if not get_bool(session.options.get(KEYBOARD_SYNC, True)):
+			xpra_args_list.append("--no-keyboard-sync")
+		if get_bool(session.options.get(READ_ONLY, False)):
+			xpra_args_list.append("--readonly")
 		window_icon = self.get_window_icon(session)
 		if window_icon:
 			xpra_args_list.append("--window-icon=%s" % window_icon)
 			if OSX:
 				xpra_args_list.append("--tray-icon=%s" % window_icon)
-		if not get_bool(session.options.get(KEYBOARD_SYNC, True)):
-			xpra_args_list.append("--no-keyboard-sync")
-		if get_bool(session.options.get(READ_ONLY, False)):
-			xpra_args_list.append("--readonly")
+		#hide the tray if preloading (only supported on posix with 0.6 onwards):
+		self.slog("pre_connected=%s, preload=%s, has min=%s" % (session.pre_connected, session.preload, self.has_min_xpra_version([0, 6])))
+		if os.name=="posix" and (session.pre_connected or session.preload) and self.has_min_xpra_version([0, 6]):
+			xpra_args_list.append("--delay-tray")
+
 		xpra_args_list += ["attach", "tcp:%s:%s" % (host, port)]
 		args = xpra_cmd(USERNAME, self.settings.xpra_command, xpra_args_list)
 		retry = (time.time() - session.start_time) < 30		#retry to connect if the session is new
@@ -151,3 +156,13 @@ class	XpraClientBase(ClientUtilBase):
 			opts[CLIPBOARD] = True
 			opts[KEYBOARD_SYNC] = True
 		return opts
+
+	def has_min_xpra_version(self, min_version):
+		if not self.settings.xpra_version:
+			return False
+		try:
+			numver = [int(x) for x in self.settings.xpra_version.split(".")]
+			return numver>=min_version
+		except:
+			pass
+		return False
