@@ -14,13 +14,13 @@ from winswitch.consts import XPRA_TYPE, X_PORT_BASE, XPRA_PORT_BASE, LOCALHOST
 from winswitch.objects.session import Session
 from winswitch.objects.server_session import ServerSession
 from winswitch.objects.server_command import ServerCommand
-from winswitch.util.common import alphanumspace, save_binary_file, delete_if_exists, xpra_path_workaround, is_xpra_version_ok, parse_version_string, no_newlines, is_valid_file
+from winswitch.util.common import alphanumspace, save_binary_file, delete_if_exists, xpra_path_workaround, parse_version_string, no_newlines, is_valid_file
 from winswitch.util.process_util import exec_nopipe, twisted_exec
 from winswitch.util.config import load_session
-from winswitch.util.file_io import get_session_password_filename, get_session_Xdummy_log_filename, get_xpra_dir
+from winswitch.util.file_io import get_session_password_filename, get_xpra_dir
 from winswitch.virt.server_util_base import ServerUtilBase
 from winswitch.virt.xpra_common import xpra_cmd
-from winswitch.virt.options_common import ENCODING, CLIPBOARD
+from winswitch.virt.options_common import ENCODING, CLIPBOARD, KEYBOARD_SYNC, JPEG_QUALITY_OPTION
 from winswitch.net.net_util import wait_for_socket
 from winswitch.util.paths import WINSWITCH_LIBEXEC_DIR
 from winswitch.util.main_loop import callLater, callFromThread
@@ -32,7 +32,7 @@ class	XpraServerUtil(ServerUtilBase):
 
 	def	__init__(self, config, add_session, remove_session, update_session_status, session_failed):
 		ServerUtilBase.__init__(self, XPRA_TYPE, XPRA_PORT_BASE, config, add_session, remove_session, update_session_status, session_failed)
-		self.prelaunch_enabled = False			#disable it until we figure out what is going wrong in some strange cases
+		self.prelaunch_enabled = True
 		self.connecting_timeout = 20			#20 seconds to go from connecting to connected
 		self.auto_upgrade_xpra_sessions = True
 		self.auto_upgrade_same_version = False
@@ -44,7 +44,7 @@ class	XpraServerUtil(ServerUtilBase):
 		self.dotxpra = DotXpra()
 		self.watched_directories.append(get_xpra_dir())			#will cause us to detect sessions when something changes in the log directory
 		self.ignored_displays = []
-		self.ignored_options_for_compare = [ENCODING, CLIPBOARD]		#these can be set client side
+		self.ignored_options_for_compare += [ENCODING, CLIPBOARD, KEYBOARD_SYNC, JPEG_QUALITY_OPTION]		#these can be set client side
 
 	def get_config_options(self):
 		return	self.get_config_options_base(detect=True, log=True, start=True)+[
@@ -100,9 +100,9 @@ class	XpraServerUtil(ServerUtilBase):
 		#Test xpra server port directly
 		return	session.port
 
-	def xmodmap_enabled(self, session, user):
+	def set_keyboard_mappings(self, session, user):
 		""" xpra does its own keymap handling """
-		return	False
+		pass
 
 
 	def	do_prepare_session_for_attach(self, session, user, disconnect, call_when_done):
@@ -148,8 +148,8 @@ class	XpraServerUtil(ServerUtilBase):
 		self.sdebug("real process=%s" % proc, xnest_display, real_command)
 
 
-	def start_display(self, session, user, is_preload, reuse_display=False):
-		self.sdebug(None, session, user, is_preload, reuse_display)
+	def start_display(self, session, user, is_preload, upgrade=False):
+		self.sdebug(None, session, user, is_preload, upgrade)
 		password_file = get_session_password_filename(session.display, session.user)
 		save_binary_file(password_file, session.password)
 		xpra_args_list = ['--bind-tcp=%s:%d' % (session.host, session.port),
@@ -164,20 +164,11 @@ class	XpraServerUtil(ServerUtilBase):
 		#JPEG_QUALITY_OPTION has no effect on server
 		#if JPEG_QUALITY_OPTION in session.options:
 		#	xpra_args_list += ["--jpeg-quality", session.options[JPEG_QUALITY_OPTION]]
-		xpra_args_list += ['start', session.display]
-		if reuse_display:
-			xpra_args_list.append('--use-display')
+		if upgrade:
+			xpra_args_list.append('upgrade')
 		else:
-			if self.config.xpra_xvfb_command:
-				xvfb = os.path.expandvars(self.config.xpra_xvfb_command)
-				if xvfb:
-					# this allows us to use the same password file for xvfb (useful when we use Xvnc as an alternative to Xvfb)
-					xvfb_bin = xvfb.replace("${password_file}", password_file)
-					if xvfb_bin.find("${logfile}")>0:
-						# set Xorg logfile:
-						logfile = get_session_Xdummy_log_filename(session.display, session.user)
-						xvfb_bin = xvfb_bin.replace("${logfile}", logfile)
-					xpra_args_list.insert(0, "--xvfb=%s" % xvfb_bin)
+			xpra_args_list.append('start')
+		xpra_args_list.append(session.display)
 		args_list = xpra_cmd(session.user, XPRA_COMMAND, xpra_args_list)
 		env = session.get_env()
 		session.xpra_version = self.config.xpra_version
@@ -188,33 +179,48 @@ class	XpraServerUtil(ServerUtilBase):
 			""" this method may be called threaded, so use callFromThread """
 			callFromThread(self.session_error, session, msg)
 
-		if line.startswith("xpra is ready."):
+		#skip date header of the form: "2012-08-20 12:28:50,489 "
+		SAMPLE="YYYY-MM-DD HH:MM:SS,mmm "
+		if line.startswith("20") and len(line)>len(SAMPLE):
+			#verify the characters are part of the date:
+			is_date_header = True
+			for i in xrange(len(SAMPLE)):
+				if line[i] not in "0123456789-:, ":
+					is_date_header = False
+					break
+			if is_date_header:
+				l = line[len(SAMPLE):]
+		else:
+			l = line
+
+		if l.startswith("xpra is ready."):
 			return	Session.STATUS_AVAILABLE
-		elif line.startswith("Handshake complete; enabling connection"):
+		elif l.startswith("Handshake complete; enabling connection") \
+			and session.status not in [Session.STATUS_CONNECTED, Session.STATUS_CONNECTING, Session.STATUS_IDLE]:
 			return	Session.STATUS_CONNECTING
-		elif line.startswith("Password matches!"):
+		elif l.startswith("Password matches!"):
 			return	Session.STATUS_CONNECTED
-		elif line.startswith("xpra client disconnected."):
+		elif l.startswith("xpra client disconnected."):
 			return	Session.STATUS_AVAILABLE
-		elif line.startswith("xpra is terminating."):
+		elif l.startswith("xpra is terminating."):
 			return	Session.STATUS_CLOSED
-		elif line.startswith("xpra: Fatal IO error"):
+		elif l.startswith("xpra: Fatal IO error"):
 			self.serror(None, session, line)
 			return	None
-		elif line.startswith("RuntimeError: could not open display"):
+		elif l.startswith("RuntimeError: could not open display"):
 			if not session.reloading and self.in_session_grace_period(session):
 				session_error("Virtual display startup error, Xpra's Xvfb may be misconfigured")
 				return	None
-		elif line.startswith("X Error of failed request"):
+		elif l.startswith("X Error of failed request"):
 			logfilename = self.get_log_file(session)
 			def poll_after_xerror():
 				self.poll_server_process(logfilename, session)
 			callLater(2, poll_after_xerror)
 			return	None
-		elif MISMATCH_RE.search(line) is not None:
-			m = MISMATCH_RE.search(line)
-			msg = line[m.end():]
-			self.serror("client version mismatch: %s" % msg, session, line)
+		elif MISMATCH_RE.search(l) is not None:
+			m = MISMATCH_RE.search(l)
+			msg = l[m.end():]
+			self.serror("client version mismatch: %s" % msg, session, l)
 			session_error("Client version mismatch: %s" % msg)
 		return	None
 
@@ -223,13 +229,20 @@ class	XpraServerUtil(ServerUtilBase):
 		We simply call "xpra stop :DISPLAY"
 		Also schedule a check 5 seconds later to see if the process is still running.
 		"""
+		#stop via unix domain socket:
 		xpra_args_list = ['stop', display]
-		try:
-			cmd = xpra_cmd(user, XPRA_COMMAND, xpra_args_list)
-			exec_nopipe(cmd)
-		except Exception, e:
-			self.serr(None, e, session, user, display)
+		#or stop via tcp socket with:
+		#xpra_args_list = ['stop', 'tcp:%s:%d' % (session.host, session.port)]
+		password_file = get_session_password_filename(session.display, session.user)
+		save_binary_file(password_file, session.password)
+		xpra_args_list.append('--password-file=%s' % password_file)
+		cmd = xpra_cmd(user, XPRA_COMMAND, xpra_args_list)
+		def display_stopped(*args):
+			self.sdebug()
+		def display_stop_error(*args):
+			self.serror(None, *args)
 			ServerUtilBase.stop_display(session, user, display)
+		twisted_exec(cmd, display_stopped, display_stop_error, log_lifecycle=False)
 
 	def detect_sessions(self):
 		"""
@@ -330,7 +343,7 @@ class	XpraServerUtil(ServerUtilBase):
 			self.slog("found session details on disk: state=%s, preload=%s" % (session.status, session.preload), state, display)
 			username = session.user
 			if username:
-				self.add_local_user(username)
+				self.add_local_user(username, None)
 		else:
 			if display in self.ignored_displays:
 				self.sdebug("unmapped display ignored", state, display)
@@ -348,7 +361,10 @@ class	XpraServerUtil(ServerUtilBase):
 				pass
 			session.name = "unknown"
 			self.slog("captured xpra session: %s" % session, state, display)
-			self.upgrade_session(session)
+			self.init_upgraded_session(session)
+			def with_session_info(*args):
+				self.start_upgraded_session(session)
+			self.grab_session_info(session, with_session_info)
 
 		self.port_mapper.add_taken_port(session.port)
 		self.config.add_session(session)
@@ -356,20 +372,63 @@ class	XpraServerUtil(ServerUtilBase):
 		if self.update_session_status:
 			self.update_session_status(session, session.status)	#ensures the controller deals with the status
 		self.firewall_util.add(session.host, session.port)
-		self.watch_existing_log(session)
 		return session
 
-	def upgrade_session(self, session):
+	def init_upgraded_session(self, session):
 		self.slog(None, session)
 		logfilename = self.get_log_file(session)
 		if session.log_watcher_cancel and not session.log_watcher_cancel.is_cancelled():
 			session.log_watcher_cancel.cancel()				#stop watching the logfile
 		if is_valid_file(logfilename):
 			os.rename(logfilename, "%s.oldserver" % logfilename)
+		session.status = Session.STATUS_STARTING
 		#must change tcp port:
 		session.port = self.get_free_port()
 		self.add_session(session)		#propagate new port
-		self.start_display(session, None, session.preload, reuse_display=True)
+
+	def start_upgraded_session(self, session):
+		self.start_display(session, None, session.preload, upgrade=True)
+		self.watch_existing_log(session)
+
+
+	def grab_session_info(self, session, callback):
+		""" Uses 'xpra info' to populate attributes and options of this session
+			Then calls callback
+		"""
+		def err_info(err):
+			self.slog(None, err)
+			callback()
+		def got_info(out):
+			self.sdebug(None, out)
+			d = {}
+			for x in out.splitlines():
+				if not x or x.startswith("#"):
+					continue
+				parts = x.split("=", 1)
+				if len(parts)==2:
+					d[parts[0]] = parts[1]
+			self.sdebug("parsed info=%s" % d, out)
+			if "encoding" in d:
+				session.options[ENCODING] = d.get("encoding")
+			session.options[CLIPBOARD] = bool(d.get("clipboard"))
+			session.options[KEYBOARD_SYNC] = bool(d.get("keyboard_sync"))
+			session.name = d.get("session_name", "")
+			session.xpra_version = parse_version_string(d.get("version"))
+			callback()
+		cmd = [self.config.xpra_command, "info", session.display]
+		twisted_exec(cmd, got_info, err_info, log_lifecycle=False)
+
+	def grab_session_version(self, session, callback):
+		def err_version(err):
+			self.serror(None, err)
+			callback()
+		def got_version(out):
+			self.sdebug(None, out)
+			session.xpra_version = parse_version_string(no_newlines(out))
+			callback()
+		cmd = [self.config.xpra_command, "version", session.display]
+		twisted_exec(cmd, got_version, err_version, log_lifecycle=False)
+
 
 	def may_upgrade_session(self, session):
 		if not self.auto_upgrade_xpra_sessions:
@@ -377,22 +436,26 @@ class	XpraServerUtil(ServerUtilBase):
 		if not self.use_gio(session):
 			self.slog("upgrading not supported without GIO support")
 			return
-		current_version = parse_version_string(self.config.xpra_version)
-		
+
+		current_version = 0
+		if getattr(self.config, "xpra_version"):
+			current_version = parse_version_string(self.config.xpra_version)
+
 		def with_version(force_upgrade):
 			self.sdebug("xpra_version=%s" % session.xpra_version, force_upgrade)
 			if not force_upgrade and session.xpra_version>=current_version:
 				self.sdebug("xpra version is up to date")
 				return
-			self.upgrade_session(session)
-		def err_version(err):
-			self.serror(None, err)
-		def got_version(out):
-			self.sdebug(None, out)
-			session.xpra_version = parse_version_string(no_newlines(out))
-			with_version(self.auto_upgrade_same_version)
+			self.init_upgraded_session(session)
+			self.start_upgraded_session(session)
+
 		if hasattr(session, "xpra_version"):
+			#already been through this, take shortcut:
 			with_version(False)
-		else:
-			cmd = [self.config.xpra_command, "version", session.display]
-			twisted_exec(cmd, got_version, err_version)
+			return
+
+		def after_getting_info(*args):
+			with_version(self.auto_upgrade_same_version)
+		def after_getting_version(*args):
+			self.grab_session_info(session, after_getting_info)
+		self.grab_session_version(session, after_getting_version)
